function [nodes_or_indices, gauss_or_coeffs] = entryId_old(N, m, ~, L, d, basis_types, return_type, funcEval, tau, gauss)
%ENTRYID_OLD performs entry identifications as described in SHT paper.
%  Supports Fourier and Chebyshev basis (also mixed).
%
%  Input:
%    N - scalar, number of possible basis indices per spatial dimension
%        (side length of cube)
%    m - scalar
%    s - currently unused
%    L - scalar, parameter for random Gaussian weights
%    d - scalar, spatial dimension
%    basis_types - cell array of length d,
%                  contains either 'fourier' or 'chebyshev'
%    return_type - either 'nodes' or indices', determines output variables,
%                  'nodes' returns pairing grids, 'indices' performs pairing
%    funcEval - (complex) double matrix: (N * m) \times 1,
%               contains function samples evaluated at required nodes,
%               only used for return_type='indices'
%    tau - scalar >= 0, threshold parameter for entry identification
%    gauss - Gaussian weights
%
%  Output:
%    nodes_or_indices - for return_type='nodes': cell array of length d
%                       containing double matrix: (N * m) \times d,
%                       for return_type='indices': cell array of length d
%                       containing column vector of detected basis indices
%    param_or_coeffs - for return_type='nodes': gauss,
%                      for return_type='indices: cell array of length d
%                      containing column vector of estimated coefficients
%
% based on SHT implementation 2018 by Bosu Choi
% version with speed improvements and generalizations 2018-2019 by Toni Volkmer

switch return_type
  case 'nodes'
    [nodes_or_indices, gauss_or_coeffs] = entryId_old_nodes(N, m, L, d, basis_types);
  case 'indices'
    [nodes_or_indices, gauss_or_coeffs] = entryId_old_indices(N, m, L, d, basis_types, funcEval, tau, gauss);
  otherwise
    error('Unknown return_type: %s', return_type);
end

end

function [indices, coeffs] = entryId_old_indices(N, m, L, d, basis_types, funcEval, tau, gauss)
  indices = cell(d,1);
  coeffs = cell(d,1);
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %all evaluations of proxy functions h_{j;k} used in the entry identification procedure
  proxy_h=zeros(d,L,N); %a matrix containing all function evaluations of the proxy functions h_{j;k} used
  for j=1:d
    funcEval_j = reshape(funcEval{j},N,m);
    for k=1:L
      for n=1:N
        proxy_h(j,k,n)=1/sqrt(m)*sum(gauss(k,:).*funcEval_j(n,:)); %evaluatation of proxy functions
      end
    end
  end
  %calculating the inner products in line 7 of Algorithm 2
  med_list=zeros(d,L,N); %a matrix containing all magnitude of the inner products in line 7 of Algorithm 2
%   med_list2=zeros(d,L,N); %a matrix containing all magnitude of the inner products in line 7 of Algorithm 2
  for  j=1:d
    switch basis_types{j}
      case 'fourier'
%         possible_indices_j = (floor(-N/2):floor(N/2))';
        % n in [-N/2,N/2]
        for k=1:L
          med_list(j,k,:)=abs(1/N*ifftshift(fft(fftshift(proxy_h(j,k,:))))); %calculating the inner products using FFT
%           for n=1:N
%             med_list2(j,k,n) = abs(1/N*reshape(proxy_h(j,k,:),1,N)*exp(-2*pi*1i*possible_indices_j(n)*(0:N-1)'/N));
%           end
        end
%         val1 = reshape(med_list(j,:,:),L,N);
%         val2 = reshape(med_list2(j,:,:),L,N);
      case 'chebyshev'
        for k=1:L     
            temp(1,:)=proxy_h(j,k,:);
            med_list(j,k,:)=abs(fct(temp)); %calculating the inner products using FCT
        end
      otherwise
        error('basis type not supported: %s', basis_types{j});
    end
  end
  %determining whether to store each entry by medianing technique
  for j=1:d
    indices{j}=[]; %entry set N_j in the paper
    coeffs{j}=[];
    switch basis_types{j}
      case 'fourier'
%         possible_indices_j = (floor(-N/2):floor((N-1)/2))';
%         k=(0:N-1)';
%         freq_basis_fourier = (-1).^k.*ceil(k/2);
%         possible_indices_j = freq_basis_fourier + ceil(N/2) + 1;
        possible_indices_j = [N-2*(0:floor(N/2)-1).'-mod(N,2); (1:2:2*floor((N+1)/2)).'] - 1;
      otherwise
        possible_indices_j = (0:N-1)';
    end
    for n=1:N
      if median(med_list(j,:,n))>tau %line 10 of Algorithm 2 using medianing
%TODO: change for Fourier (indices from [-N,N-1])
        indices{j}=[indices{j}; possible_indices_j(n)]; %store each entry in N_j with nonzero median value
        coeffs{j}=[coeffs{j}; median(med_list(j,:,n))];
      end
    end
  end
end

function [nodes, gauss] = entryId_old_nodes(N, m, L, d, basis_types)
  nodes = cell(d,1);
  gauss = randn(L,m);
  %randomly chosen X_j's: m by d (differs slightly from SHT paper)
  x = basis_gen_rnd_nodes(basis_types, m);
  for j = 1:d
    basis_type_j = basis_types{j};
    switch basis_type_j
      case 'fourier'
        %sample points U_j in the paper : 1 by N
        xprime = (0:(N-1))'/N;
      case 'chebyshev'
        xprime = -cos(pi/N*((0:N-1)+1/2))';
      otherwise
        error('Unknown basis_type: %s', basis_type_j);
    end

    xxprime_j = nan(N,m,d);
    for k=1:N
      xxprime_j(k,:,:) = x;
    end
    xxprime_j(:,:,j) = repmat(xprime,1,m);
    nodes{j} = reshape(xxprime_j,N*m,d);
  end
end
