function [nodes_or_indices, wx_or_coeffs] = entryId_quad_new(N, m, ~, d, basis_types, return_type, funcEval, tau, wx)
%ENTRYID_QUAD_NEW performs entry identifications using quadrature formulas.
%  Supports Fourier, (preconditioned) Legendre, and Chebyshev basis
%    (also mixed bases).
%
%  Input:
%    N - scalar, number of possible basis indices per spatial dimension
%        (side length of cube)
%    m - scalar
%    s - currently unused
%    d - scalar, spatial dimension
%    basis_types - cell array of length d,
%                  contains either 'fourier' or 'chebyshev'
%    return_type - either 'nodes' or indices', determines output variables,
%                  'nodes' returns pairing grids, 'indices' performs pairing
%    funcEval - (complex) double matrix: (L_prime * m) \times 1,
%               contains function samples evaluated at required nodes,
%               only used for return_type='indices'
%    tau - scalar >= 0, threshold parameter for entry identification
%  Output:
%    nodes_or_indices - for return_type='nodes': cell array of length d
%                       containing double matrix: (L_prime * m) \times d,
%                       for return_type='indices': cell array of length d
%                       containing column vector of detected basis indices
%    param_or_coeffs - for return_type='nodes': gauss,
%                      for return_type='indices: cell array of length d
%                      containing column vector of estimated coefficients
%
% version with speed improvements and generalizations 2018-2019 by Toni Volkmer

switch return_type
  case 'nodes'
    [nodes_or_indices, wx_or_coeffs] = entryId_quad_new_nodes(N, m, d, basis_types);
  case 'indices'
    [nodes_or_indices, wx_or_coeffs] = entryId_quad_new_indices(N, m, d, basis_types, funcEval, tau, wx);
  otherwise
    error('Unknown return_type: %s', return_type);
end

end

function [indices, coeffs] = entryId_quad_new_indices(N, m, d, basis_types, funcEval, tau, wx)
  indices = cell(d,1);
  coeffs = cell(d,1);
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  for j=1:d
    if isempty(wx{j}{2})
      L_prime = N;
    else
      L_prime = length(wx{j}{1});
    end
    funcEval_j = reshape(funcEval{j},L_prime,m);
    coefficients_j = nan(N, m);
    switch basis_types{j}
      case 'fourier'
        % n in [-N/2,N/2]
        for ell=1:m
          coefficients_j(:,ell) = 1/N*ifftshift(fft(fftshift( funcEval_j(:,ell) )));
        end
      case 'chebyshev'
        for ell=1:m
          coefficients_j(:,ell) = fct( funcEval_j(:,ell) );
        end
      case {'legendre','legendre_preconditioned'}
        basis_values = basis_eval({'legendre'}, (0:N-1)', wx{j}{2});
        if strcmp(basis_types{j}, 'legendre_preconditioned')
          funcEval_j = (pi/2)^(-1/2) * (1-wx{j}{2}.^2).^(-1/4) .* funcEval_j;
        end
        for n=0:N-1
          coefficients_j(n+1,:) = sum(repmat(wx{j}{1}.*basis_values(:,n+1), 1, m) .* funcEval_j,1);
        end
      otherwise
        error('basis type not supported: %s', basis_types{j});
    end
    coefficients_j = sqrt(sum(1/m*abs(coefficients_j).^2,2));
    ind_j = find(coefficients_j > tau);
    %%%%%%%%%%%%%%%%%%%%%%
    switch basis_types{j}
      case 'fourier'
        possible_indices_j = [N-2*(0:floor(N/2)-1).'-mod(N,2); (1:2:2*floor((N+1)/2)).'] - 1;
      otherwise
        possible_indices_j = (0:N-1)';
    end
    indices{j} = possible_indices_j(ind_j); %entry set N_j in the paper
    coeffs{j} = coefficients_j(ind_j);
  end
end

function [nodes, wx] = entryId_quad_new_nodes(N, m, d, basis_types)
  nodes = cell(d,1);
  wx = cell(d,1);
  %randomly chosen X_j's: m by d (differs slightly from SHT paper)
  x = basis_gen_rnd_nodes(basis_types, m);
  for j = 1:d
    basis_type_j = basis_types{j};
    wx{j} = cell(1,2);
    switch basis_type_j
      case 'fourier'
        %sample points U_j in the paper : 1 by N
        xprime = (0:(N-1))'/N;
        Lprime = N;
      case 'chebyshev'
        xprime = -cos(pi/N*((0:N-1)+1/2))';
        Lprime = N;
      case {'legendre','legendre_preconditioned'}
        [xprime, w] = gauss_legendre_nodes_weights(N-1);
        wx{j}{1} = w;
        wx{j}{2} = xprime;
        Lprime = length(xprime);
        clear w;
      otherwise
        error('Unknown basis_type: %s', basis_type_j);
    end
    xxprime_j = nan(Lprime,m,d);
    for k=1:Lprime
      xxprime_j(k,:,:) = x;
    end
    xxprime_j(:,:,j) = repmat(xprime,1,m);
    nodes{j} = reshape(xxprime_j,Lprime*m,d);
  end
end

function [nodes, weights] = gauss_legendre_nodes_weights(n)
%Source: GAUSS_LEGENDRE_INTEGRATE_FAST applies a Gauss-Legendre quadrature rule.
%https://people.sc.fsu.edu/~jburkardt/m_src/quadrule_fast/gauss_legendre_integrate_fast.m
%Original author: Lloyd Trefethen
%Reference: Lloyd Trefethen, Is Gauss Quadrature Better than Clenshaw-Curtis?
%           SIAM Review, Volume 50, Number 1, 2008, pages 67-87.
  beta = 0.5 ./ sqrt ( 1 - ( 2 * ( 1:n ) ) .^ (-2) );

  tridiag = diag ( beta, 1 ) + diag ( beta, -1 );

  [ eigval, eigvec ] = eig ( tridiag );

  nodes = diag ( eigvec );
  [ nodes, i ] = sort ( nodes );
  nodes = nodes(:);
%   weights = 2 * eigval(1,i) .^ 2;
  weights = eigval(1,i) .^ 2;
  weights = weights(:);
end