function [output,coeff_est]=pairing(basis_types, entrysets, sampleP, funcEval, s) 
%PAIRING pairing part of Support identification in Algorithm 2.
%  Supports Fourier, (preconditioned) Legendre, and Chebyshev basis
%    (also mixed bases).
%  Energy estimate computation itself is without for loop now.
%
%  Input:
%    basis_types - cell array of length d,
%                  contains either 'fourier' or 'chebyshev' or 'legendre'
%    entrysets - cell array of size 1 by d containing 1 by (about)2s vectors,
%                N_j's from entry identification
%    sampleP - cell array of size d-1 by m_1 containing 1 by jj /jj=2,...,d,
%              sampling points W_j={w_{j,\ell}}_{\ell in [m_1]} in the paper
%    funcEval - samples in multidimensional array of size m_1 by m_2 by d-1,
%               v^P in the paper 
%    s - sparsity
%
%  Output:
%    output - detected basis indices of size s by d
%    coeff_est - absolute value of the corresponding coefficients
%
% based on SHT implementation 2018 by Bosu Choi
% version with speed improvements and generalizations 2018-2019 by Toni Volkmer

d = length(entrysets);

%updating P_1 in the paper
P = entrysets{1}(:,1);
%concatinating the entries 
for j=2:1:d
  funcEval_j = funcEval{j-1};
  m_1 = size(funcEval_j,1);
  m_2 = size(funcEval_j,2);
    leng1=size(P,1);  % size of the previous P_j in the paper
    leng2=size(entrysets{j},1); %(about s), size of N_{j+1} in the paper
    leng=leng1*leng2; %(about s^2), size of P_j X N_{j+1}
    basis=zeros(leng,j); %(about s^2) by j, initilization of candidate prefixes from P_j X N_{j+1}
    % creating P_j X N_{j+1}
    for ind1=1:leng1
        basis((ind1-1)*leng2+1:ind1*leng2,:) = [repmat(P(ind1,:),leng2,1)  entrysets{j}(:,1)];
    end
    % evaluating basis functions induced by P_j X N_{j+1} at each w_{j+1,ell}
    sampleP_j = sampleP{j-1};
    basisEval = basis_eval(basis_types(1:j), basis, sampleP_j);
    basisEval = basisEval.';
    energy_est = sum((abs(conj(basisEval(:,:))*funcEval_j(:,:))).^2, 2);
    energy_est=energy_est/(m_1*m_1*m_2); %normalization
    % sorting to get the updated P_j
    [sortE,I]=sort(energy_est,'descend');%sorting according to the size of E_{j,(\tilde{n}, ...)}
    if leng<s
        ss=leng; % for the case where leng is smaller than s 
    else
        ss=s; 
    end    
    P = basis(I(1:ss),:);
end
coeff_est = sqrt(sortE(1:ss)); % estimation of the magnitude of coefficients c_n
output = P;